@@ -2,7 +2,7 @@ from __future__ import print_function, division, absolute_import
 
 from .config import config
 from .core import connect, rpc
-from .deploy import LocalCluster
+from .deploy import LocalCluster, Adaptive
 from .diagnostics import progress
 from .client import (Client, Executor, CompatibleExecutor,
                      wait, as_completed, default_client, fire_and_forget,
