@@ -615,6 +615,21 @@ def test_retire_workers(c, s, a, b):
     assert not workers
 
 
+@gen_cluster(client=True)
+def test_retire_workers_n(c, s, a, b):
+    yield s.retire_workers(n=1)
+    assert len(s.workers) == 1
+
+    yield s.retire_workers(n=0)
+    assert len(s.workers) == 1
+
+    yield s.retire_workers(n=1)
+    assert len(s.workers) == 0
+
+    yield s.retire_workers(n=0)
+    assert len(s.workers) == 0
+
+
 @gen_cluster(client=True, ncores=[('127.0.0.1', 1)] * 4)
 def test_workers_to_close(cl, s, *workers):
     s.task_duration['a'] = 4
@@ -721,7 +736,7 @@ def test_file_descriptors(c, s):
 @gen_cluster(client=True)
 def test_learn_occupancy(c, s, a, b):
     futures = c.map(slowinc, range(1000), delay=0.01)
-    while not any(ts.who_has for ts in s.tasks.values()):
+    while sum(len(ts.who_has) for ts in s.tasks.values()) < 10:
         yield gen.sleep(0.01)
 
     assert 1 < s.total_occupancy < 40
